<?php
/*
Plugin Name: Feedfabrik Blogbook Printing
Plugin URI: http://www.feedfabrik.com/blog/wordpress-printing-plugin
Version: 1.10
Author: feedfabrik
Author URI: http://www.feedfabrik.com
Description: Provide the Wordpress admin interface with a new button in the Tools section that allows printing of the blog content as a blogbook at <a href="http://www.feedfabrik.com">Feedfabrik</a>. Blogbooks can be downloaded as free PDF or can be ordered in print. The plugin interfaces directly with feedfabrik.com and provides a secure and easy way to start the book preparation process without providing a username or password to <a href="http://www.feedfabrik.com">feedfabrik.com</a>.
License: GPL2
*/

/*  Copyright 2010, 2011, 2012  Feedfabrik Media Services  (email : support@feedfabrik.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// Make sure we don't expose any info if called directly
if ( !function_exists( 'add_action' ) ) {
	echo "Hi there!  I'm just a plugin, not much I can do when called directly.";
	exit;
}

// here comes the real stuff
if (!class_exists('Feedfabrik')) {
	
	// class definition
	class Feedfabrik {
		
		// Constructor
		function Feedfabrik() {}
		
		// static variables - workaround for PHP4
		function get_static() {
			return array(
				'version'           => '1.10',
				'user_options_name' => 'feedfabrik_options',
				'feedfabrik_url'    => 'https://www.feedfabrik.com',
				'expiry'            => 172800
			);
		}
		
		// get_user_meta wrapper for backwards compatibility
		function wrap_get_user_meta($user_id, $key, $single) {
			if (function_exists('get_user_meta')) {
				return get_user_meta($user_id, $key, $single);
			} else {
				return get_usermeta($user_id, $key);
			}
		}
		
		// update_user_meta wrapper for backwards compatibility
		function wrap_update_user_meta($user_id, $meta_key, $meta_value) {
			if (function_exists('update_user_meta')) {
				return update_user_meta($user_id, $meta_key, $meta_value);
			} else {
				return update_usermeta($user_id, $meta_key, $meta_value);
			}
		}
		
		// update_delete_meta wrapper for backwards compatibility
		function wrap_delete_user_meta($user_id, $meta_key) {
			if (function_exists('delete_user_meta')) {
				return delete_user_meta($user_id, $meta_key);
			} else {
				return delete_usermeta($user_id, $meta_key);
			}
		}
		
		// get_site_url wrapper for backwards compatibility
		function wrap_get_site_url() {
			if (function_exists('get_site_url')) {
				return get_site_url();
			} else {
				return site_url();
			}
		}
		
		// Determine the location
		function get_plugin_path() {
			return plugins_url('', __FILE__).'/';
		}

		// set user options
		function get_user_options() {
			// current user
			global $current_user;
			$feedfabrik_static = Feedfabrik::get_static();
			
			// default values
			$user_options = array(
				'auth_key' => md5(uniqid("",true)),
				'timestamp' => time()
			);

			// retrieve previously set options
			$user_options_retrieved = $this->wrap_get_user_meta($current_user->ID, $feedfabrik_static['user_options_name'], true);
			
			if (empty($user_options_retrieved)) {
				$this->wrap_update_user_meta($current_user->ID, $feedfabrik_static['user_options_name'], $user_options);
			} else {
				$last_timestamp = $user_options_retrieved['timestamp'];
				
				// timestamp is only valid for one day
				if (empty($last_timestamp) || ($user_options['timestamp']-$last_timestamp) > $feedfabrik_static['expiry']) {
					$this->wrap_update_user_meta($current_user->ID, $feedfabrik_static['user_options_name'], $user_options);
				} else {
					$user_options = $user_options_retrieved;
				}
			}

			return $user_options;
		}

		// activate plugin and init user options
		function activate() {
			$this->get_user_options();
		}
		
		// deactivate plugin and cleanup
		function deactivate() {
			$feedfabrik_static = Feedfabrik::get_static();
			
			// loop over all users and remove their plugin meta data
			$blogusers = get_users_of_blog();
			foreach ($blogusers as $user) {
				$this->wrap_delete_user_meta($user->ID, $feedfabrik_static['user_options_name']);
			}
		}
		
		// print out blogprinting page
		function print_blog_page() {
			// current user
			global $current_user;
			
			$feedfabrik_static = Feedfabrik::get_static();
			
			// get settings
			$user_options = $this->get_user_options();
			
			// offer the print button
			?>
			<div class=wrap>
				<div id="icon-tools" class="icon32"></div>
				<h2>Feedfabrik Blogbook Printing</h2>
				<form method="post" action="<?php echo $feedfabrik_static['feedfabrik_url']; ?>/bookfabrik/connect-wp">
					<table style="padding: 20px;"><tr>
						<td valign="bottom" style="padding-right: 20px; padding-bottom: 3px;"><img src="<?php echo $this->get_plugin_Path(); ?>ffountain.png"></td>
						<td valign="bottom">Click below to proceed to the <a href="<?php echo $feedfabrik_static['feedfabrik_url']; ?>">Feedfabrik website</a><br />
							to preview your blog for printing or PDF download.</br></br>
							Please be patient, the import can take some time for large blogs.<br/>
							If you have a problem, <a href="http://www.feedfabrik.com/support">please contact us for help</a>.</td>
					</tr></table>
					<input type="hidden" name="user_id" value="<?php echo $current_user->user_login; ?>"/>
					<input type="hidden" name="auth_key" value="<?php echo $user_options['auth_key']; ?>"/>
					<input type="hidden" name="blog_url" value="<?php echo $this->wrap_get_site_url(); ?>"/>
					<input type="hidden" name="plugin_version" value="<?php echo $feedfabrik_static['version']; ?>"/>
					<p class="submit">
						<input type="submit" class="button-primary" name="print_blog" value="<?php _e('Print my Blog', 'Feedfabrik') ?>" />
					</p>
					<iframe src="https://www.facebook.com/plugins/like.php?href=http%3A%2F%2Fwww.feedfabrik.com&amp;layout=standard&amp;show_faces=true&amp;width=200&amp;action=like&amp;colorscheme=light" scrolling="no" frameborder="0" allowTransparency="true" style="border:none; overflow:hidden; width:300px; height:100px;"></iframe>
				</form>
			</div>
			<?php				
		}

		// setup admin panel
		function setup_admin_panel() {
			global $dl_feedfabrik;
			if (!isset($dl_feedfabrik)) {
				return;
			}
			if (function_exists('add_management_page')) {
				add_management_page('Feedfabrik Blogbook Printing', 'Print', 0, basename(__FILE__), array(&$dl_feedfabrik, 'print_blog_page'));
			}
		}
		
		// register Feedfabrik API
		function add_filter($methods) {
			// put ourselves into the list of XML-RPC methods
			$methods['feedfabrik.getUserInfo'] = array('Feedfabrik','get_user_info');
			$methods['feedfabrik.getUsersBlogs'] = array('Feedfabrik','get_users_blogs');
			$methods['feedfabrik.getRecentPostTitles'] = array('Feedfabrik','get_recent_post_titles');
			$methods['feedfabrik.getPostIds'] = array('Feedfabrik','get_post_ids');
			$methods['feedfabrik.getRecentPosts'] = array('Feedfabrik','get_recent_posts');
			$methods['feedfabrik.getPosts'] = array('Feedfabrik','get_posts');
			$methods['feedfabrik.getVersion'] = array('Feedfabrik','get_version');
			$methods['feedfabrik.ping'] = array('Feedfabrik','ping');
			return $methods;
		}
		
		// get user info
		function get_user_info($args) {
			global $wp_xml_rpc_server;
			
			Feedfabrik::escape($args);
	
			$username = $args[1];
			$password = $args[2];
	
			$user = Feedfabrik::login($username, $password);
			
			if (is_a($user,'IXR_Error')) return $user;
				
			$struct = array(
				'nickname'  => $user->nickname,
				'userid'    => $user->ID,
				'url'       => $user->user_url,
				'lastname'  => $user->last_name,
				'firstname' => $user->first_name
			);
	
			return $struct;
		}
		
		// get users blogs
		function get_users_blogs($args) {
			global $current_site;
			
			Feedfabrik::escape($args);

			$username = $args[0];
			$password = $args[1];
	
			$user = Feedfabrik::login($username, $password);
			
			if (is_a($user,'IXR_Error')) return $user;
	
			// If this isn't on WPMU
			if ( !function_exists('is_multisite') || !is_multisite() ) {
				$is_admin = current_user_can('manage_options');
		
				$struct = array(
					'isAdmin'  => $is_admin,
					'url'      => get_option('home') . '/',
					'blogid'   => '1',
					'blogName' => get_option('blogname'),
					'xmlrpc'   => site_url( 'xmlrpc.php' )
				);
		
				return array($struct);
			}
	
			$blogs = (array) get_blogs_of_user( $user->ID );
			$struct = array( );
	
			foreach ( $blogs as $blog ) {
				// Don't include blogs that aren't hosted at this site
				if ( $blog->site_id != $current_site->id )
					continue;
	
				$blog_id = $blog->userblog_id;
				switch_to_blog($blog_id);
				$is_admin = current_user_can('manage_options');
	
				$struct[] = array(
					'isAdmin'		=> $is_admin,
					'url'			=> get_option( 'home' ) . '/',
					'blogid'		=> $blog_id,
					'blogName'		=> get_option( 'blogname' ),
					'xmlrpc'		=> site_url( 'xmlrpc.php' )
				);
	
				restore_current_blog( );
			}
	
			return $struct;
		}
			
		// get post IDs
		function get_post_ids($args) {
			global $wpdb;
	
			Feedfabrik::escape($args);
				
			$blog_ID   = (int) $args[0];
			$username  = $args[1];
			$password  = $args[2];
	
			$user = Feedfabrik::login($username, $password);
			
			if (is_a($user,'IXR_Error')) return $user;
				
			$post_ids = $wpdb->get_col($wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE post_parent=0 AND post_type='post'"));
	
			if ( $post_ids === FALSE )
				return array( );
				
			return $post_ids;
		}
		
		// get recent post titles
		function get_recent_post_titles($args) {
	
			Feedfabrik::escape($args);
				
			$blog_ID   = (int) $args[0];
			$username  = $args[1];
			$password  = $args[2];
			$query = array( 'numberposts' => absint( $args[3] ) );
	
			$user = Feedfabrik::login($username, $password);
			
			if (is_a($user,'IXR_Error')) return $user;
				
			$posts_list = wp_get_recent_posts( $query );
				
			if ( !$posts_list )
				return array( );
				
			foreach ($posts_list as $entry) {
	
				$post_date = mysql2date('Ymd\TH:i:s', $entry['post_date'], false);
				$post_date_gmt = mysql2date('Ymd\TH:i:s', $entry['post_date_gmt'], false);
	
				// For drafts use the GMT version of the date
				if ( $entry['post_status'] == 'draft' )
					$post_date_gmt = get_gmt_from_date( mysql2date( 'Y-m-d H:i:s', $entry['post_date'] ), 'Ymd\TH:i:s' );
	
				$struct[] = array(
					'dateCreated' => new IXR_Date($post_date),
					'userid' => $entry['post_author'],
					'postid' => (string) $entry['ID'],
					'title' => $entry['post_title'],
					'post_status' => $entry['post_status'],
					'date_created_gmt' => new IXR_Date($post_date_gmt)
				);
	
			}
	
			$recent_posts = array();
			for ( $j=0; $j<count($struct); $j++ ) {
				array_push($recent_posts, $struct[$j]);
			}
	
			return $recent_posts;
		}
		
		// get recent posts
		function get_recent_posts($args) {

			Feedfabrik::escape($args);
	
			$blog_ID   = (int) $args[0];
			$username  = $args[1];
			$password  = $args[2];
			$query     = array( 'numberposts' => absint( $args[3] ) );
	
			$user = Feedfabrik::login($username, $password);
			
			if (is_a($user,'IXR_Error')) return $user;
				
			$posts_list = wp_get_recent_posts( $query );
	
			if ( !$posts_list )
				return array( );
	
			foreach ($posts_list as $entry) {
				// put the entry into the global post variable as object to make it available to the filters
				$GLOBALS["post"] = (object) $entry;
	
				$post_date = mysql2date('Ymd\TH:i:s', $entry['post_date'], false);
				$post_date_gmt = mysql2date('Ymd\TH:i:s', $entry['post_date_gmt'], false);
	
				// For drafts use the GMT version of the date
				if ( $entry['post_status'] == 'draft' )
					$post_date_gmt = get_gmt_from_date( mysql2date( 'Y-m-d H:i:s', $entry['post_date'] ), 'Ymd\TH:i:s' );
	
				$categories = array();
				$catids = wp_get_post_categories($entry['ID']);
				foreach( $catids as $catid )
					$categories[] = get_cat_name($catid);
	
				$tagnames = array();
				$tags = wp_get_post_tags( $entry['ID'] );
				if ( !empty( $tags ) ) {
					foreach ( $tags as $tag ) {
						$tagnames[] = $tag->name;
					}
					$tagnames = implode( ', ', $tagnames );
				} else {
					$tagnames = '';
				}
	
				$post = get_extended($entry['post_content']);
				$link = post_permalink($entry['ID']);
				
				// apply filters
				$main     = apply_filters('the_content', $post['main']);
				$extended = apply_filters('the_content', $post['extended']);
				$title    = apply_filters('the_title_rss', $entry['post_title']);
	
				// Get the post author info.
				$author = get_userdata($entry['post_author']);
	
				$allow_comments = ('open' == $entry['comment_status']) ? 1 : 0;
				$allow_pings = ('open' == $entry['ping_status']) ? 1 : 0;
	
				// Consider future posts as published
				if ( $entry['post_status'] === 'future' )
					$entry['post_status'] = 'publish';
	
				// Get post format
				if (function_exists('get_post_format')) {
					$post_format = get_post_format( $entry['ID'] );
				}
				if ( empty( $post_format ) )
					$post_format = 'standard';

				$struct[] = array(
					'dateCreated' => new IXR_Date($post_date),
					'userid' => $entry['post_author'],
					'postid' => (string) $entry['ID'],
					'description' => $main,
					'title' => $title,
					'link' => $link,
					'permaLink' => $link,
					// commented out because no other tool seems to use this
					// 'content' => $entry['post_content'],
					'categories' => $categories,
					'mt_excerpt' => $entry['post_excerpt'],
					'mt_text_more' => $extended,
					'mt_allow_comments' => $allow_comments,
					'mt_allow_pings' => $allow_pings,
					'mt_keywords' => $tagnames,
					'wp_slug' => $entry['post_name'],
					'wp_password' => $entry['post_password'],
					'wp_author_id' => $author->ID,
					'wp_author_display_name' => $author->display_name,
					'date_created_gmt' => new IXR_Date($post_date_gmt),
					'post_status' => $entry['post_status'],
					'custom_fields' => Feedfabrik::get_custom_fields($entry['ID']),
					'wp_post_format' => $post_format
				);
			}
	
			$recent_posts = array();
			for ( $j=0; $j<count($struct); $j++ ) {
				array_push($recent_posts, $struct[$j]);
			}
	
			return $recent_posts;
		}
		
		// get posts
		function get_posts($args) {

			Feedfabrik::escape($args);
	
			$blog_ID  = (int) $args[0];
			$username = $args[1];
			$password = $args[2];
			$post_ids = $args[3];
	
			$user = Feedfabrik::login($username, $password);
			
			if (is_a($user,'IXR_Error')) return $user;
				
			foreach ($post_ids as $id) {
				
				$post_ID = (int) $id;
				$entry = wp_get_single_post($id, ARRAY_A);
				
				// put the entry into the global post variable as object to make it available to the filters
				$GLOBALS["post"] = (object) $entry;
				
				$post_date = mysql2date('Ymd\TH:i:s', $entry['post_date'], false);
				$post_date_gmt = mysql2date('Ymd\TH:i:s', $entry['post_date_gmt'], false);
	
				// For drafts use the GMT version of the date
				if ( $entry['post_status'] == 'draft' )
					$post_date_gmt = get_gmt_from_date( mysql2date( 'Y-m-d H:i:s', $entry['post_date'] ), 'Ymd\TH:i:s' );
	
				$categories = array();
				$catids = wp_get_post_categories($entry['ID']);
				foreach( $catids as $catid )
					$categories[] = get_cat_name($catid);
	
				$tagnames = array();
				$tags = wp_get_post_tags( $entry['ID'] );
				if ( !empty( $tags ) ) {
					foreach ( $tags as $tag ) {
						$tagnames[] = $tag->name;
					}
					$tagnames = implode( ', ', $tagnames );
				} else {
					$tagnames = '';
				}
	
				$post = get_extended($entry['post_content']);
				$link = post_permalink($entry['ID']);
				
				// apply filters
				$main     = apply_filters('the_content', $post['main']);
				$extended = apply_filters('the_content', $post['extended']);
				$title    = apply_filters('the_title_rss', $entry['post_title']);
					
				// Get the post author info.
				$author = get_userdata($entry['post_author']);
	
				$allow_comments = ('open' == $entry['comment_status']) ? 1 : 0;
				$allow_pings = ('open' == $entry['ping_status']) ? 1 : 0;
	
				// Consider future posts as published
				if ( $entry['post_status'] === 'future' )
					$entry['post_status'] = 'publish';
	
				// Get post format
				if (function_exists('get_post_format')) {
					$post_format = get_post_format( $entry['ID'] );
				}
				if ( empty( $post_format ) )
					$post_format = 'standard';
					
				$struct[] = array(
					'dateCreated' => new IXR_Date($post_date),
					'userid' => $entry['post_author'],
					'postid' => (string) $entry['ID'],
					'description' => $main,
					'title' => $title,
					'link' => $link,
					'permaLink' => $link,
					// commented out because no other tool seems to use this
					// 'content' => $entry['post_content'],
					'categories' => $categories,
					'mt_excerpt' => $entry['post_excerpt'],
					'mt_text_more' => $extended,
					'mt_allow_comments' => $allow_comments,
					'mt_allow_pings' => $allow_pings,
					'mt_keywords' => $tagnames,
					'wp_slug' => $entry['post_name'],
					'wp_password' => $entry['post_password'],
					'wp_author_id' => $author->ID,
					'wp_author_display_name' => $author->display_name,
					'date_created_gmt' => new IXR_Date($post_date_gmt),
					'post_status' => $entry['post_status'],
					'custom_fields' => Feedfabrik::get_custom_fields($entry['ID']),
					'wp_post_format' => $post_format
				);
			}
	
			$recent_posts = array();
			for ( $j=0; $j<count($struct); $j++ ) {
				array_push($recent_posts, $struct[$j]);
			}
	
			return $recent_posts;
		}
		
		// version info
		function get_version($args) {
			$feedfabrik_static = Feedfabrik::get_static();
			$version = $feedfabrik_static['version'];
			return $version;
		}			
		
		// ping
		function ping($args) {
			return $args;
		}
					
		// helper function
		function get_custom_fields($post_id) {
			$post_id = (int) $post_id;
	
			$custom_fields = array();
	
			foreach ( (array) has_meta($post_id) as $meta ) {
				// Don't expose protected fields.
				if ( strpos($meta['meta_key'], '_wp_') === 0 ) {
					continue;
				}
	
				$custom_fields[] = array(
					"id"    => $meta['meta_id'],
					"key"   => $meta['meta_key'],
					"value" => $meta['meta_value']
				);
			}
	
			return $custom_fields;
		}
		
		// escape the XML-RPC arguments
		function escape(&$array) {
			global $wpdb;
			
			// switch off time limit, if we are allowed to
			if ( function_exists('set_time_limit') ) { 
				@set_time_limit(0);
			}
	
			if (!is_array($array)) {
				return($wpdb->escape($array));
			} else {
				foreach ( (array) $array as $k => $v ) {
					if ( is_array($v) ) {
						Feedfabrik::escape($array[$k]);
					} else if ( is_object($v) ) {
						//skip
					} else {
						$array[$k] = $wpdb->escape($v);
					}
				}
			}
		}

		// login to get user
		function login($username, $password) {
			$feedfabrik_static = Feedfabrik::get_static();
			
			// make sure the data is safe
			$username = sanitize_user($username);
			$password = trim($password);

			// data is missing?
			if ( empty($username) || empty($password) ) {
				return new IXR_Error(403, __('Bad login/pass combination.'));
			}
			
			// the plugin uses the user ID in the username field, so try if we have such a user
			$userdata = get_user_by('login', $username);
			if ( !$userdata )
				return new IXR_Error(403, __('User unknown.'));
			
			// we don't want spammers
			if ( function_exists('is_multisite') && is_multisite() ) {
				// Is user marked as spam?
				if ( 1 == $userdata->spam)
					return new IXR_Error(403, __('Account has been marked as spammer.'));
						
				// Is a user's blog marked as spam?
				if ( function_exists('is_super_admin') && !is_super_admin( $userdata->ID ) && isset($userdata->primary_blog) ) {
					$details = get_blog_details( $userdata->primary_blog );
					if ( is_object( $details ) && $details->spam == 1 )
						return new IXR_Error(403, __('Site has been suspended.'));
				}
			}
			
			// ok, there is a user, first try to see if we have a real password
			$user = wp_authenticate_username_password(null, $username, $password);
			if (is_a($user, 'WP_User'))
				return $user;
				
			// now the normal password didn't work, so try the token instead
			$user_options = $userdata->feedfabrik_options;
			if (empty($user_options)) {
				return new IXR_Error(403, __('Feedfabrik plugin not properly activated.'));
			}
			$token = $user_options['auth_key'];
			if (empty($token)) {
				return new IXR_Error(403, __('Feedfabrik plugin did not register a token.'));
			}
			if ($token != $password) {
				return new IXR_Error(403, __('Access token invalid: ' . $token . ' vs ' . $password));
			}
			
			// and is the token recent enough?
			$timestamp = $user_options['timestamp'];
			if (empty($timestamp)) {
				return new IXR_Error(403, __('Access token has invalid timestamp.'));
			}
			if (time()-$timestamp > $feedfabrik_static['expiry']) {
				return new IXR_Error(403, __('Access token expired.'));
			}
			
			// ok, let the user in
			$user = new WP_User($userdata->ID);

			return $user;
		}	
	}

	// initialize class
	$dl_feedfabrik = new Feedfabrik();

	// setup actions and filters
	if (isset($dl_feedfabrik)) {
		// actions
		register_activation_hook(__FILE__, array(&$dl_feedfabrik, 'activate'));
		register_deactivation_hook(__FILE__,  array(&$dl_feedfabrik, 'deactivate'));
		add_action('admin_menu', array(&$dl_feedfabrik, 'setup_admin_panel'));
		
		// filters
		add_filter('xmlrpc_methods', array(&$dl_feedfabrik, 'add_filter'));
	}
}
?>